@@ -2,7 +2,7 @@
 # debianbts.py - Routines to deal with the debbugs web pages
 #
 #   Written by Chris Lawrence <lawrencc@debian.org>
-#   (C) 1999-2005 Chris Lawrence
+#   (C) 1999-2006 Chris Lawrence
 #
 # This program is freely distributable per the following license:
 #
@@ -22,13 +22,28 @@
 #
 # Version ##VERSION##; see changelog for revision history
 #
-# $Id: debianbts.py,v 1.18 2005/04/30 06:19:48 lawrencc Exp $
+# $Id: debianbts.py,v 1.24.2.15 2008-04-18 05:38:27 lawrencc Exp $
 
 import sgmllib, glob, os, re, reportbug, rfc822, time, urllib, checkversions
+import textwrap
+from reportbug_exceptions import *
 from urlutils import open_url
-from types import StringTypes
 import sys
 
+import mailbox
+import email
+import email.Errors
+import cStringIO
+import cgi
+
+def msgfactory(fp):
+    try:
+        return email.message_from_file(fp)
+    except email.Errors.MessageParseError:
+        # Don't return None since that will
+        # stop the mailbox iterator
+        return ''
+
 class Error(Exception):
     pass
 
@@ -38,7 +53,7 @@ SEVERITIES = {
     whole system) break, or causes serious data loss, or introduces a
     security hole on systems where you install the package.""",
     'grave' : """makes the package in question unusable by most or all users,
-    or causes data loss, or introduces a security hole allowing access 
+    or causes data loss, or introduces a security hole allowing access
     to the accounts of users who use the package.""",
     'serious' : """is a severe violation of Debian policy (that is,
     the problem is a violation of a 'must' or 'required' directive);
@@ -135,30 +150,36 @@ def convert_severity(severity, type='debbugs'):
 debother = {
     'base' : 'General bugs in the base system',
 # Actually a real package, but most people don't have boot-floppies installed for good reason
-    'boot-floppy' : '(Obsolete, please use boot-floppies instead.)',
-    'boot-floppies' : 'Bugs in the woody installation subsystem',
+#    'boot-floppy' : '(Obsolete, please use boot-floppies instead.)',
+#    'boot-floppies' : 'Bugs in the woody installation subsystem',
     'bugs.debian.org' : 'The bug tracking system, @bugs.debian.org',
     'cdimage.debian.org' : 'CD Image issues',
     'cdrom' : 'Problems with installation from CD-ROMs',
 # dpkg-iwj -- The dpkg branch maintained by Ian Jackson
     'debian-policy' : 'Proposed changes in the Debian policy documentation',
-    'ftp.debian.org' : 'Problems with the FTP site',
+    'ftp.debian.org' : 'Problems with the FTP site and Package removal requests',
     'general' : 'General problems (e.g., that many manpages are mode 755)',
-    'install' : 'Problems with the sarge installer.',
-    'installation' : 'General installation problems not covered otherwise.',
-    'kernel' : '(Obsolete, please use "kernel-image" instead.)',
-    'kernel-image' : 'Problems with the Linux kernel, or the kernel shipped with Debian',
+#    'install' : 'Problems with the sarge installer.',
+#    'installation' : 'General installation problems not covered otherwise.',
+    'installation-reports' : 'Problems with installing Debian',
+    'kernel' : 'IMPORTANT: Obsolete, please use "linux-image" instead (linux-image will be used in any case)',
+    'linux-image' : 'Problems with the Linux kernel, or the kernel shipped with Debian',
     'listarchives' :  'Problems with the WWW mailing list archives',
     'lists.debian.org' : 'The mailing lists, debian-*@lists.debian.org.',
     'mirrors' : 'Problems with Debian archive mirrors.',
-    'nonus.debian.org' : 'Problems with the non-US FTP site',
+    'nm.debian.org' : 'New Maintainer process and nm.debian.org website',
     'press' : 'Press release issues',
-    'project' : 'Problems related to Project administration',
+    'project' : 'Problems related to project administration',
     'qa.debian.org' : 'Problems related to the quality assurance group',
-#slink-cd -- Slink CD 
-#spam -- Spam (reassign spam to here so we can complain about it)
+#slink-cd -- Slink CD
+    'release.debian.org' : 'Requests regarding Debian releases and release team tools',
+    'release-notes' : 'Problems with the release notes',
+    'security-tracker' : 'The Debian Security Bug Tracker',
     'security.debian.org' : 'Problems with the security updates server',
+    'spam' : 'Spam (reassign spam to here so we can complain about it)',
+    'tech-ctte' : 'Issues to be referred to the technical committee',
     'upgrade-reports' : 'Reports of successful and unsucessful upgrades',
+    'wiki.debian.org' : 'Problems with the Debian wiki',
     'wnpp' : 'Work-Needing and Prospective Packages list',
     'www.debian.org' : 'Problems with the WWW site (including other *.debian.org sites)'
     }
@@ -167,12 +188,143 @@ progenyother = {
     'debian-general' : 'Any non-package-specific bug',
     }
 
-def handle_wnpp(package, bts, ui, fromaddr, online=True, http_proxy=None):
-    desc = body = ''
+def handle_debian_ftp(package, bts, ui, fromaddr, online=True, http_proxy=None):
+    body = reason = archs = ''
+    suite = 'unstable'
     headers = []
     pseudos = []
     query = True
-    
+
+    tag = ui.menu('What sort of request is this?  (If none of these '
+                  'things mean anything to you, or you are trying to report '
+                  'a bug in an existing package, please press Enter to '
+                  'exit reportbug.)', {
+        'ROM' :
+        "Package removal - Request Of Maintainer.",
+        'RoQA' :
+        "Package removal - Requested by the QA team.",
+        'ROP' :
+        "Package removal - Request of Porter.",
+        'ROSRM' :
+        "Package removal - Request of Stable Release Manager.",
+        'NBS' :
+        "Package removal - Not Built [by] Source.",
+        'NPOASR' :
+        "Package removal - Never Part Of A Stable Release.",
+        'NVIU' :
+        "Package removal - Newer Version In Unstable.",
+        'ANAIS' :
+        "Package removal - Architecture Not Allowed In Source.",
+        'ICE' :
+        "Package removal - Internal Compiler Error.",
+        'other' :
+        "Not a package removal request, report other problems.",
+        }, 'Choose the request type: ', empty_ok=True)
+    if not tag:
+        ui.long_message('To report a bug in a package, use the name of the package, not ftp.debian.org.\n')
+        raise SystemExit
+
+    severity = 'normal'
+    if tag == 'other':
+        return
+    else:
+        prompt = 'Please enter the name of the package: '
+        package = ui.get_string(prompt)
+        if not package:
+            ui.ewrite('You seem to want to report a generic bug, not request a removal\n')
+            return
+
+        ui.ewrite('Checking status database...\n')
+        info = reportbug.get_package_status(package)
+        available = info[1]
+
+        query = False
+        if not available:
+            info = reportbug.get_source_package(package)
+            if info:
+                info = reportbug.get_package_status(info[0][0])
+
+        if not info:
+            cont = ui.select_options(
+                "This package doesn't appear to exist; continue?",
+                'yN', {'y': 'Ignore this problem and continue.',
+                       'n': 'Exit without filing a report.' })
+            if cont == 'n':
+                sys.exit(1)
+        else:
+            package = info[12] or package
+
+        suite = ui.menu('Is the removal to be done in a suite other than'
+                        ' "unstable"?  Please press Enter for "unstable"', {
+            'oldstable' : "Old stable.",
+            'oldstable-proposed-updates' : "Old stable proposed updates.",
+            'stable' : "Stable.",
+            'stable-proposed-updates' : "Stable proposed updates.",
+            'testing' : "Testing",
+	    'testing-proposed-updates' : "Testing proposed updates",
+            'experimental' : "Experimental.",
+        }, 'Choose the suite: ', empty_ok=True)
+        if not suite:
+            suite = 'unstable'
+
+        if suite not in ('testing', 'unstable', 'experimental'):
+            headers.append('X-Debbugs-CC: debian-release@lists.debian.org')
+            ui.ewrite('Your report will be carbon-copied to debian-release.\n')
+
+        why = 'Please enter the reason for removal: '
+        reason = ui.get_string(why)
+        if not reason: return
+
+        partial = ui.select_options(
+            "Is this removal request for specific architectures?",
+            'yN', {'y': 'This is a partial (specific architectures) removal.',
+                   'n': 'This removal is for all architectures.' })
+        if partial == 'y':
+            prompt = 'Please enter the arch list separated by a space: '
+            archs = ui.get_string(prompt)
+            if not archs:
+                ui.long_message('Partial removal requests must have a list of architectures.\n')
+                raise SystemExit
+
+    if archs:
+        if suite != 'unstable':
+            subject = 'RM: %s/%s [%s] -- %s; %s' % (package, suite, archs, tag, reason)
+        else:
+            subject = 'RM: %s [%s] -- %s; %s' % (package, archs, tag, reason)
+    else:
+        if suite != 'unstable':
+            subject = 'RM: %s/%s -- %s; %s' % (package, suite, tag, reason)
+        else:
+            subject = 'RM: %s -- %s; %s' % (package, tag, reason)
+
+    if suite == 'testing':
+        ui.ewrite('Please use the following subject and send a mail to '
+                  'debian-release@lists.debian.org to request removal.\n')
+        ui.ewrite(subject)
+        sys.exit(1)
+
+    return (subject, severity, headers, pseudos, body, query)
+
+
+itp_template = textwrap.dedent(u"""\
+    * Package name    : %(package)s
+      Version         : x.y.z
+      Upstream Author : Name <somebody@example.org>
+    * URL             : http://www.example.org/
+    * License         : (GPL, LGPL, BSD, MIT/X, etc.)
+      Programming Lang: (C, C++, C#, Perl, Python, etc.)
+      Description     : %(short_desc)s
+
+    (Include the long description here.)
+    """)
+
+
+def handle_wnpp(package, bts, ui, fromaddr, online=True, http_proxy=None):
+    short_desc = body = ''
+    headers = []
+    pseudos = []
+    query = True
+
     tag = ui.menu('What sort of request is this?  (If none of these '
                   'things mean anything to you, or you are trying to report '
                   'a bug in an existing package, please press Enter to '
@@ -180,9 +332,9 @@ def handle_wnpp(package, bts, ui, fromaddr, online=True, http_proxy=None):
         'O' :
         "The package has been `Orphaned'. It needs a new maintainer as soon as possible.",
         'RFA' :
-        "This is a `Request for Adoption'. Due to lack of time, resources, interest or something similar, the current maintainer is asking for someone else to maintain this package. He/she will maintain it in the meantime, but perhaps not in the best possible way. In short: the package needs a new maintainer.",
+        "This is a `Request for Adoption'. Due to lack of time, resources, interest or something similar, the current maintainer is asking for someone else to maintain this package. They will maintain it in the meantime, but perhaps not in the best possible way. In short: the package needs a new maintainer.",
         'RFH' :
-        "This is a `Request For Help'. The current maintainer wants to continue to maintain this package, but he/she needs some help to do this, because his/her time is limited or the package is quite big and needs several maintainers.",
+        "This is a `Request For Help'. The current maintainer wants to continue to maintain this package, but they needs some help to do this, because their time is limited or the package is quite big and needs several maintainers.",
         'ITP' :
         "This is an `Intent To Package'. Please submit a package description along with copyright and URL in such a report.",
         'RFP' :
@@ -215,30 +367,22 @@ def handle_wnpp(package, bts, ui, fromaddr, online=True, http_proxy=None):
                 'warning message may have been produced in error.',
                 'Exit without filing a report.', default=0):
                 sys.exit(1)
-            
+
         severity = 'wishlist'
 
-        desc = ui.get_string(
+        short_desc = ui.get_string(
             'Please briefly describe this package; this should be an '
             'appropriate short description for the eventual package: ')
-        if not desc:
+        if not short_desc:
             return
 
         if tag == 'ITP':
             headers.append('X-Debbugs-CC: debian-devel@lists.debian.org')
-            pseudos.append('Owner: %s' % fromaddr)
+            pseudos.append(u'Owner: %s' % fromaddr.decode('utf-8', 'replace'))
             ui.ewrite('Your report will be carbon-copied to debian-devel, '
                       'per Debian policy.\n')
 
-        body = """* Package name    : %s
-  Version         : x.y.z
-  Upstream Author : Name <somebody@example.org>
-* URL             : http://www.example.org/
-* License         : (GPL, LGPL, BSD, MIT/X, etc.)
-  Description     : %s
-
-(Include the long description here.)
-""" % (package, desc)
+        body = itp_template % vars()
     elif tag in ('O', 'RFA', 'RFH'):
         severity = 'normal'
         query = False
@@ -254,11 +398,11 @@ def handle_wnpp(package, bts, ui, fromaddr, online=True, http_proxy=None):
                        'n': 'Exit without filing a report.' })
             if cont == 'n':
                 sys.exit(1)
-            desc = fulldesc = ''
+            short_desc = long_desc = ''
         else:
-            desc = info[10] or ''
-            package = info[11] or package
-            fulldesc = info[12]
+            short_desc = info[11] or ''
+            package = info[12] or package
+            long_desc = info[13]
 
         if tag == 'O' and info and info[9] in \
                ('required', 'important', 'standard'):
@@ -269,19 +413,19 @@ def handle_wnpp(package, bts, ui, fromaddr, online=True, http_proxy=None):
             ui.ewrite('Your request will be carbon-copied to debian-devel, '
                       'per Debian policy.\n')
 
-        if fulldesc:
+        if long_desc:
             orphstr = 'intend to orphan'
             if tag == 'RFA':
                 orphstr = 'request an adopter for'
             elif tag == 'RFH':
                 orphstr = 'request assistance with maintaining'
-                
+
             body = ('I %s the %s package.\n\n'
                     'The package description is:\n') % (orphstr, package)
-            body = body + fulldesc + '\n'
-        
-    if desc:
-        subject = '%s: %s -- %s' % (tag, package, desc)
+            body = body + long_desc + '\n'
+
+    if short_desc:
+        subject = '%s: %s -- %s' % (tag, package, short_desc)
     else:
         subject = '%s: %s' % (tag, package)
 
@@ -292,25 +436,16 @@ def handle_wnpp(package, bts, ui, fromaddr, online=True, http_proxy=None):
 SYSTEMS = { 'debian' :
             { 'name' : 'Debian', 'email': '%s@bugs.debian.org',
               'btsroot' : 'http://www.debian.org/Bugs/',
-              'otherpkgs' : debother, 'nonvirtual' : ['kernel-image'],
-              'specials' : { 'wnpp': handle_wnpp },
+              'otherpkgs' : debother,
+              'nonvirtual' : ['linux-image', 'kernel-image'],
+              'specials' :
+                { 'wnpp': handle_wnpp,
+                  'ftp.debian.org': handle_debian_ftp },
               # Dependency packages
               'deppkgs' : ('gcc', 'g++', 'cpp', 'gcj', 'gpc', 'gobjc',
                            'chill', 'gij', 'g77', 'python', 'python-base',
                            'x-window-system-core', 'x-window-system'),
               'cgiroot' : 'http://bugs.debian.org/cgi-bin/' },
-            'kde' :
-            { 'name' : 'KDE Project', 'email': '%s@bugs.kde.org',
-              'btsroot': 'http://bugs.kde.org/' },
-            'mandrake' :
-            { 'name' : 'Linux-Mandrake', 'email': '%s@bugs.linux-mandrake.com',
-              'type' : 'mailto', 'query-dpkg' : False },
-            'gnome' :
-            { 'name' : 'GNOME Project', 'email': '%s@bugs.gnome.org',
-              'type' : 'mailto', 'query-dpkg' : False },
-            'ximian' :
-            { 'name' : 'Ximian', 'email': '%s@bugs.ximian.com',
-              'type' : 'mailto' },
             'progeny' :
             { 'name' : 'Progeny', 'email' : 'bugs@progeny.com',
               'type' : 'gnats', 'otherpkgs' : progenyother },
@@ -320,14 +455,8 @@ SYSTEMS = { 'debian' :
             'guug' :
             { 'name' : 'GUUG (German Unix User Group)',
               'email' : '%s@bugs.guug.de', 'query-dpkg' : False },
-            'grml' :
-            { 'name' : 'grml', 'email': '%s@bugs.grml.org',
-              'btsroot' : 'http://bugs.grml.org/',
-              'cgiroot' : 'http://bugs.grml.org/cgi-bin/' },
             }
 
-SYSTEMS['helixcode'] = SYSTEMS['ximian']
-
 CLASSES = {
     'sw-bug' : 'The problem is a bug in the software or code.  For'
     'example, a crash would be a sw-bug.',
@@ -353,15 +482,13 @@ TAGS = {
 ##    'woody' : 'This bug only applies to the woody release (Debian 3.0).',
 ##    'sarge' : 'This bug only applies to the sarge release (Debian 3.1).',
 ##    'sid' : 'This bug only applies to the unstable branch of Debian.',
-    'experimental' : 'This bug only applies to a package in the experimental '
-    'branch of Debian.',
     "l10n" : "This bug reports a localization/internationalization issue.",
 ##    'done' : 'No more tags.',
     }
 
 EXTRA_TAGS = ['potato', 'woody', 'sarge', 'security', 'sid', 'upstream']
 
-TAGLIST = ['l10n', 'patch', 'experimental']
+TAGLIST = ['l10n', 'patch']
 CRITICAL_TAGLIST = ['security']
 
 def yn_bool(setting):
@@ -375,30 +502,36 @@ def yn_bool(setting):
 def cgi_report_url(system, number, archived=False, mbox=False):
     root = SYSTEMS[system].get('cgiroot')
     if root:
-        return '%sbugreport.cgi?bug=%d&archive=%s&mbox=%s' % (
+        return '%sbugreport.cgi?bug=%d&archived=%s&mbox=%s' % (
             root, number, archived, yn_bool(mbox))
     return None
 
 def cgi_package_url(system, package, archived=False, source=False,
-                    repeatmerged=True):
+                    repeatmerged=True, version=None):
     root = SYSTEMS[system].get('cgiroot')
     if not root: return None
-    
-    package = urllib.quote_plus(package.lower())
-    qtype = "pkg"
-    if source:
-        qtype = "src"
-        
-    repeat = yn_bool(repeatmerged)
-    archive = yn_bool(archived)
 
-    return '%spkgreport.cgi?%s=%s&archive=%s&repeatmerged=%s&show_list_header=no&show_list_footer=no' % (root, qtype, package, archive, repeat)
+    #package = urllib.quote_plus(package.lower())
+    if source:
+        query = {'src' : package.lower()}
+    else:
+        query = {'pkg' : package.lower()}
+
+    query['repeatmerged'] = yn_bool(repeatmerged)
+    query['archived'] = yn_bool(archived)
+
+    if version:
+        query['version'] = str(version)
+
+    qstr = urllib.urlencode(query)
+    #print qstr
+    return '%spkgreport.cgi?%s' % (root, qstr)
 
 def package_url(system, package, mirrors=None, source=False,
                 repeatmerged=True):
     btsroot=get_btsroot(system, mirrors)
     package = urllib.quote_plus(package.lower())
-    return btsroot+('db/pa/l%s.html' % package) 
+    return btsroot+('db/pa/l%s.html' % package)
 
 def report_url(system, number, mirrors=None):
     number = str(number)
@@ -480,6 +613,8 @@ class BTSParser(sgmllib.SGMLParser):
         self.mode = mode
         self.cgi = cgi
         self.followups = followups
+        self.inbuglist = self.intrailerinfo = False
+        self.bugtitle = None
         if followups:
             self.preblock = []
         else:
@@ -491,7 +626,7 @@ class BTSParser(sgmllib.SGMLParser):
 
     def handle_data(self, data):
         if self.savedata is not None:
-            self.savedata = self.savedata + data
+            self.savedata += data
 
     # --- Hooks to save data; shouldn't need to be overridden
 
@@ -500,19 +635,11 @@ class BTSParser(sgmllib.SGMLParser):
 
     def save_end(self, mode=False):
         data = self.savedata
+        if not mode and data:
+            data = ' '.join(data.split())
         self.savedata = None
-        if not mode: data = ' '.join(data.split())
         return data
 
-    def check_li(self):
-        if self.mode == 'summary':
-            data = self.save_end()
-            if data:
-                self.lidatalist.append(data)
-                self.bugcount = self.bugcount + 1
-
-            self.lidata = False
-
     def start_h1(self, attrs):
         self.save_bgn()
         self.oldmode = self.mode
@@ -529,17 +656,61 @@ class BTSParser(sgmllib.SGMLParser):
 
     def end_h2(self):
         if self.mode == 'summary':
-            self.hierarchy.append( (self.save_end(), []) )
+            hiertitle = self.save_end()
+            if 'bug' in hiertitle:
+                self.hierarchy.append( (hiertitle, []) )
         self.endh2 = True # We are at the end of a title, flag <pre>
 
-    def do_br(self, attrs):
-        if self.lidata and self.mode == 'summary': self.check_li()
+    def start_ul(self, attrs):
+        if self.mode == 'summary':
+            for k, v in attrs:
+                if k == 'class' and v == 'bugs':
+                    self.inbuglist = True
 
+    def end_ul(self):
+        if self.inbuglist:
+            self.check_li()
+
+        self.inbuglist = False
+
+    def do_br(self, attrs):
         if self.mode == 'title':
             self.savedata = ""
-        
-    def do_li(self, attrs):
+        elif self.mode == 'summary' and self.inbuglist and not self.intrailerinfo:
+            self.bugtitle = self.save_end()
+            self.intrailerinfo = True
+            self.save_bgn()
+
+    def check_li(self):
         if self.mode == 'summary':
+            if not self.intrailerinfo:
+                self.bugtitle = self.save_end()
+                trailinfo = ''
+            else:
+                trailinfo = self.save_end()
+
+            match = re.search(r'fixed:\s+([\w.+~-]+(\s+[\w.+~:-]+)?)', trailinfo)
+            if match:
+                title = self.bugtitle
+                bits = re.split(r':\s+', title, 1)
+                if len(bits) > 1:
+                    buginfo = '%s [FIXED %s]: %s' % (
+                        bits[0], match.group(1), bits[1])
+                else:
+                    if title.endswith(':'):
+                        title = title[:-1]
+
+                    buginfo = '%s [FIXED %s]' % (title, match.group(1))
+            else:
+                buginfo = self.bugtitle
+
+            self.lidatalist.append(buginfo)
+            self.bugcount += 1
+
+            self.lidata = self.intrailerinfo = False
+
+    def do_li(self, attrs):
+        if self.mode == 'summary' and self.inbuglist:
             if self.lidata: self.check_li()
 
             self.lidata = True
@@ -555,7 +726,7 @@ class BTSParser(sgmllib.SGMLParser):
             if not self.endh2: return
         else:
             if self.cgi and self.preblock: return
-        
+
         self.save_bgn()
 
     def end_pre(self):
@@ -570,16 +741,55 @@ class BTSParser(sgmllib.SGMLParser):
         elif not (self.preblock and self.cgi):
             self.preblock = self.save_end(1)
 
+    def reorganize(self):
+        if not self.hierarchy:
+            return
+
+        newhierarchy = []
+        fixed = []
+        fixedfinder = re.compile(r'\[FIXED ([^\]]+)\]')
+        resolvedfinder = re.compile(r'Resolved')
+
+        for (title, buglist) in self.hierarchy:
+            if 'Resolved' in title:
+                newhierarchy.append( (title, buglist) )
+                continue
+
+            bugs = []
+            for bug in buglist:
+                if fixedfinder.search(bug):
+                    fixed.append(bug)
+                else:
+                    bugs.append(bug)
+
+            if bugs:
+                title = ' '.join(title.split()[:-2])
+                if len(bugs) != 1:
+                    title += ' (%d bugs)' % len(bugs)
+                else:
+                    title += ' (1 bug)'
+
+                newhierarchy.append( (title, bugs) )
+
+        if fixed:
+            self.hierarchy = [('Bugs fixed in subsequent releases (%d bugs)' % len(fixed), fixed)] + newhierarchy
+
 def parse_html_report(number, url, http_proxy, followups=False, cgi=True):
     page = open_url(url, http_proxy)
     if not page:
         return None
 
-    content = page.read()
     parser = BTSParser(cgi=cgi, followups=followups)
-    parser.feed(content)
+    for line in page:
+        parser.feed(line)
     parser.close()
 
+    try:
+        page.fp._sock.recv = None
+    except:
+        pass
+    page.close()
+
     items = parser.preblock
     title = "#%d: %s" % (number, parser.title)
 
@@ -606,29 +816,103 @@ def parse_html_report(number, url, http_proxy, followups=False, cgi=True):
 
     return (title, output)
 
+# XXX: Need to handle charsets properly
+def parse_mbox_report(number, url, http_proxy, followups=False):
+    page = open_url(url, http_proxy)
+    if not page:
+        return None
+
+    # Make this seekable
+    wholefile = cStringIO.StringIO(page.read())
+
+    try:
+        page.fp._sock.recv = None
+    except:
+        pass
+    page.close()
+
+    mbox = mailbox.UnixMailbox(wholefile, msgfactory)
+    title = ''
+
+    output = []
+    for message in mbox:
+        if not message:
+            pass
+
+        subject = message.get('Subject')
+        if not title:
+            title = subject
+
+        date = message.get('Date')
+        fromhdr = message.get('From')
+
+        body = entry = ''
+        for part in message.walk():
+            if part.get_content_type() == 'text/plain' and not body:
+                body = part.get_payload(None, True)
+
+        if fromhdr:
+            entry += 'From: %s%s' % (fromhdr, os.linesep)
+
+        if subject and subject != title:
+            entry += 'Subject: %s%s' % (subject, os.linesep)
+
+        if date:
+            entry += 'Date: %s%s' % (date, os.linesep)
+
+        if entry:
+            entry += os.linesep
+
+        entry += body.rstrip('\n') + os.linesep
+
+        output.append(entry)
+
+    if not output:
+        return None
+
+    title = "#%d: %s" % (number, title)
+    return (title, output)
+
 def get_cgi_reports(package, system='debian', http_proxy='', archived=False,
-                    source=False):
-    page = open_url(cgi_package_url(system, package, archived, source),
-                    http_proxy)
+                    source=False, version=None):
+    try:
+        page = open_url(cgi_package_url(system, package, archived, source,
+                                    version=version), http_proxy)
+    except:
+        raise NoNetwork
+
     if not page:
         return (0, None, None)
 
-    content = page.read()
-    if 'Maintainer' not in content:
-        return (0, None, None)
-    
-    parser = BTSParser(cgi=True)
-    parser.feed(content)
-    parser.close()
+    #content = page.read()
+    #if 'Maintainer' not in content:
+    #    return (0, None, None)
 
-    return parser.bugcount, parser.title, parser.hierarchy
+    parser = BTSParser(cgi=True)
+    for line in page:
+        parser.feed(line)
+    parser.close()
+    try:
+        page.fp._sock.recv = None
+    except:
+        pass
+    page.close()
+
+    # Reorganize hierarchy to put recently-fixed bugs at top
+    parser.reorganize()
+
+    data = (parser.bugcount, parser.title, parser.hierarchy)
+    del parser
+
+    return data
 
 def get_cgi_report(number, system='debian', http_proxy='', archived=False,
                    followups=False):
     number = int(number)
 
-    url = cgi_report_url(system, number, archived='no')
-    return parse_html_report(number, url, http_proxy, followups, cgi=True)
+    url = cgi_report_url(system, number, archived='no', mbox=True)
+    return parse_mbox_report(number, url, http_proxy, followups)
+    #return parse_html_report(number, url, http_proxy, followups, cgi=True)
 
 def get_btsroot(system, mirrors=None):
     if mirrors:
@@ -638,26 +922,38 @@ def get_btsroot(system, mirrors=None):
                 return alternates[mirror]
     return SYSTEMS[system].get('btsroot', '')
 
-def get_reports(package, system='debian', mirrors=None,
+def get_reports(package, system='debian', mirrors=None, version=None,
                 http_proxy='', archived=False, source=False):
-    if isinstance(package, StringTypes):
+    if isinstance(package, basestring):
         if SYSTEMS[system].get('cgiroot'):
-            result = get_cgi_reports(package, system, http_proxy, archived,
-                                     source)
+            try:
+                result = get_cgi_reports(package, system, http_proxy, archived,
+                                     source, version=version)
+            except:
+                raise NoNetwork
             if result: return result
 
         url = package_url(system, package, mirrors, source)
-        page = open_url(url, http_proxy)
+        try:
+            page = open_url(url, http_proxy)
+        except:
+            raise NoNetwork
         if not page:
             return (0, None, None)
 
-        content = page.read()
-        if 'Maintainer' not in content:
-            return (0, None, None)
+        #content = page.read()
+        #if 'Maintainer' not in content:
+        #    return (0, None, None)
 
         parser = BTSParser()
-        parser.feed(content)
+        for line in page:
+            parser.feed(line)
         parser.close()
+        try:
+            page.fp._sock.recv = None
+        except:
+            pass
+        page.close()
 
         return parser.bugcount, parser.title, parser.hierarchy
 
@@ -671,7 +967,7 @@ def get_reports(package, system='debian', mirrors=None,
             title, body = result
             this_hierarchy.append(title)
             #print title
-    
+
     title = "Multiple bug reports"
     bugcount = len(this_hierarchy)
     hierarchy = [('Reports', this_hierarchy)]
@@ -682,10 +978,22 @@ def get_report(number, system='debian', mirrors=None,
                http_proxy='', archived=False, followups=False):
     number = int(number)
     if SYSTEMS[system].get('cgiroot'):
-        result = get_cgi_report(number, system, http_proxy, archived,followups)
+        result = get_cgi_report(number, system, http_proxy, archived,
+                                followups)
         if result: return result
-        
+
     url = report_url(system, number, mirrors)
     if not url: return None
 
     return parse_html_report(number, url, http_proxy, followups, cgi=False)
+
+class NullParser(sgmllib.SGMLParser):
+    def __init__(self):
+        sgmllib.SGMLParser.__init__(self)
+
+if __name__ == '__main__':
+    import pprint
+
+    data = get_cgi_reports('reportbug')
+    pprint.pprint(data)
+    time.sleep(1000)
