@@ -1,6 +1,6 @@
 # Text user interface for reportbug
 #   Written by Chris Lawrence <lawrencc@debian.org>
-#   (C) 2001-04 Chris Lawrence
+#   (C) 2001-06 Chris Lawrence
 #
 # This program is freely distributable per the following license:
 #
@@ -18,12 +18,11 @@
 ##  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 ##  SOFTWARE.
 #
-# $Id: reportbug_ui_text.py,v 1.14 2005/05/10 22:25:10 lawrencc Exp $
+# $Id: reportbug_ui_text.py,v 1.19.2.9 2008-04-18 05:38:28 lawrencc Exp $
 
-import commands, sys, os, re, math, string, debianbts, errno
+import commands, sys, os, re, math, string, debianbts, errno, reportbug
 from reportbug_exceptions import *
 from urlutils import launch_browser
-from types import StringTypes
 import dircache
 import glob
 import getpass
@@ -34,6 +33,7 @@ except:
     from optik import textwrap
 
 ISATTY = sys.stdin.isatty()
+charset = 'us-ascii'
 
 try:
     r, c = commands.getoutput('stty size').split()
@@ -45,10 +45,17 @@ def ewrite(message, *args):
     if not ISATTY:
         return
 
-    sys.stderr.write(message % args)
+    if args:
+        message = message % args
+
+    if isinstance(message, unicode):
+        message = message.encode(charset, 'replace')
+
+    sys.stderr.write(message)
     sys.stderr.flush()
 
 log_message = ewrite
+display_failure = ewrite
 
 def indent_wrap_text(text, starttext='', indent=0, linelen=None):
     """Wrapper for textwrap.fill to the existing API."""
@@ -194,21 +201,46 @@ def yes_no(msg, yeshelp, nohelp, default=True, nowrap=False):
     return (res == 'y')
 
 def long_message(text, *args):
-    ewrite(indent_wrap_text(text % args))
+    if args:
+        ewrite(indent_wrap_text(text % args))
+    else:
+        ewrite(indent_wrap_text(text))
+
+final_message = long_message
 
 def get_string(prompt, options=None, title=None, force_prompt=False,
                default='', completer=None):
     if prompt and (len(prompt) < 2*columns/3) and not force_prompt:
         if default:
             prompt = '%s [%s]: ' % (prompt, default)
-            return our_raw_input(prompt, options, completer) or default
-        return our_raw_input(prompt, options, completer)
+            response = our_raw_input(prompt, options, completer) or default
+        else:
+            response = our_raw_input(prompt, options, completer)
     else:
         if prompt:
             ewrite(indent_wrap_text(prompt))
         if default:
-            return our_raw_input('[%s]> ' % default, options, completer) or default
-        return our_raw_input('> ', options, completer)
+            response = our_raw_input('[%s]> ' % default, options, completer) or default
+        else:
+            response = our_raw_input('> ', options, completer)
+
+    # Translate the response into a Unicode string
+    if response is not None:
+        response = unicode(response, charset, 'replace')
+
+    return response
+
+def get_multiline(prompt):
+    ewrite('\n')
+    ewrite(indent_wrap_text(prompt + "  Press ENTER on a blank line to continue.\n"))
+    l = []
+    while 1:
+        entry = get_string('', force_prompt=True).strip()
+        if not entry:
+            break
+        l.append(entry)
+    ewrite('\n')
+    return l
 
 def get_password(prompt=None):
     return getpass.getpass(prompt)
@@ -247,7 +279,7 @@ def menu(par, options, prompt, default=None, title=None, any_ok=False,
 
     ewrite(indent_wrap_text(par, linelen=columns)+'\n')
 
-    if type(options) == type({}):
+    if isinstance(options, dict):
         options = options.copy()
         # Convert to a list
         if order:
@@ -380,10 +412,7 @@ def show_report(number, system, mirrors,
                     raise
         skip_pager = False
 
-        if queryonly:
-            options = 'Orbq'
-        else:
-            options = 'xOrbq'
+        options = 'xOrbq'
             
         if (current_message+1) < len(messages):
             options = 'N'+options.lower()
@@ -419,7 +448,7 @@ def show_report(number, system, mirrors,
 
 def handle_bts_query(package, bts, mirrors=None, http_proxy="",
                      queryonly=False, title="", screen=None, archived='no',
-                     source=False):
+                     source=False, version=None):
     import debianbts
     
     root = debianbts.SYSTEMS[bts].get('btsroot')
@@ -432,7 +461,7 @@ def handle_bts_query(package, bts, mirrors=None, http_proxy="",
     if source:
         srcstr = " (source)"
         
-    if isinstance(package, StringTypes):
+    if isinstance(package, basestring):
         long_message('Querying %s BTS for reports on %s%s...\n',
                      debianbts.SYSTEMS[bts]['name'], package, srcstr)
     else:
@@ -443,19 +472,20 @@ def handle_bts_query(package, bts, mirrors=None, http_proxy="",
     bugs = []
     try:
         (count, title, hierarchy)=debianbts.get_reports(
-            package, bts, mirrors=mirrors,
+            package, bts, mirrors=mirrors, version=version,
             source=source, http_proxy=http_proxy, archived=archived)
         if debianbts.SYSTEMS[bts].has_key('namefmt'):
             package2 = debianbts.SYSTEMS[bts]['namefmt'] % package
             (count2, title2, hierarchy2) = \
                      debianbts.get_reports(package2, bts,
                                            mirrors=mirrors, source=source,
-                                           http_proxy=http_proxy)
+                                           http_proxy=http_proxy,
+                                           version=version)
             count = count+count2
             for entry in hierarchy2:
                 hierarchy.append( (package2+' '+entry[0], entry[1]) )
 
-        exp = re.compile(r'\#(\d+):')
+        exp = re.compile(r'#(\d+)[ :]')
         for entry in hierarchy or []:
             for bug in entry[1]:
                 match = exp.match(bug)
@@ -475,8 +505,9 @@ def handle_bts_query(package, bts, mirrors=None, http_proxy="",
         return browse_bugs(hierarchy, count, bugs, bts, queryonly,
                            mirrors, http_proxy, screen, title)
 
-    except IOError:
-        res = select_options('Unable to connect to BTS; continue', 'yN',
+    except (IOError, NoNetwork):
+        ewrite('Unable to connect to %s BTS; ', debianbts.SYSTEMS[bts]['name'])
+        res = select_options('continue', 'yN',
                              {'y': 'Keep going.',
                               'n': 'Abort.'})
         if res == 'n':
@@ -647,7 +678,7 @@ def proc_hierarchy(hierarchy):
 
     # Copy & paste from handle_bts_query()
     bugs = []
-    exp = re.compile(r'\#(\d+):')
+    exp = re.compile(r'\#(\d+)[ :]')
     for entry in hierarchy or []:
 	for bug in entry[1]:
 	    match = exp.match(bug)
@@ -675,6 +706,7 @@ def search_bugs(hierarchyfull, bts, queryonly, mirrors,
 	return "FilterEnd"
         
     count, bugs = proc_hierarchy(hierarchy)
+    exp = re.compile(r'\#(\d+):')
     
     if not count:
 	our_raw_input('No match found, press ENTER to continue.')
@@ -835,3 +867,69 @@ def search_bugs(hierarchyfull, bts, queryonly, mirrors,
             lastpage.append('\n')
             scount = scount + 1
     return "FilterEnd"
+
+def display_report(text, use_pager=True):
+    if not use_pager:
+        ewrite(text)
+        return
+
+    pager = os.environ.get('PAGER', 'sensible-pager')
+    try:
+        os.popen(pager, 'w').write(text)
+    except IOError:
+        pass
+
+def spawn_editor(message, filename, editor, charset='utf-8'):
+    if not editor:
+        ewrite('No editor found!\n')
+        return (message, 0)
+
+    edname = os.path.basename(editor.split()[0])
+
+    # Move the cursor for lazy buggers like me; add your editor here...
+    ourline = 0
+    for (lineno, line) in enumerate(file(filename)):
+        if line == '\n' and not ourline:
+            ourline = lineno + 2
+        elif line.strip() == reportbug.NEWBIELINE:
+            ourline = lineno + 2
+
+    opts = ''
+    if 'vim' in edname:
+        # Force *vim to edit the file in the foreground, instead of forking
+        opts = '-f '
+
+    if ourline:
+        if edname in ('vi', 'nvi', 'vim', 'elvis', 'gvim', 'kvim'):
+            opts = '-c :%d' % ourline
+        elif (edname in ('elvis-tiny', 'gnuclient', 'ee', 'pico', 'nano', 'zile') or
+              'emacs' in edname):
+            opts = '+%d' % ourline
+        elif edname in ('jed', 'xjed'):
+            opts = '-g %d' % ourline
+        elif edname == 'kate':
+            opts = '--line %d' % ourline
+
+    if '&' in editor or edname == 'kate':
+        ewrite("Spawning %s in background; please press Enter when done "
+               "editing.\n", edname)
+    else:
+        ewrite("Spawning %s...\n", edname)
+
+    result = os.system("%s %s '%s'" % (editor, opts, filename))
+
+    if result:
+        ewrite('Warning: possible error exit from %s: %d\n', edname, result)
+
+    if not os.path.exists(filename):
+        ewrite('Bug report file %s removed!', filename)
+        sys.exit(1)
+
+    if '&' in editor: return (None, 1)
+
+    newmessage = file(filename).read().decode(charset, 'replace')
+
+    if newmessage == message:
+        ewrite('No changes were made in the editor.\n')
+
+    return (newmessage, newmessage != message)
